@@ -2,7 +2,7 @@
 
 
 package Lab::Bus::IsoBus;
-our $VERSION = '3.11';
+our $VERSION = '3.20';
 
 use strict;
 use Lab::Connection;
@@ -18,6 +18,8 @@ our @ISA = ("Lab::Bus");
 our %fields = (
 	type => 'IsoBus',
 	base_connection => undef,
+	IsoEnableTermChar=>1,
+	IsoTermChar=>"\r",
 	brutal => 0,	# brutal as default?
 	wait_status=>10e-6, # sec;
 	wait_query=>10e-6, # sec;
@@ -42,33 +44,21 @@ sub new {
 		}
 		else {
 			# TODO implement twin detection
-			# $Lab::Bus::BusList{$self->type()}->{'default'} = $self;
-			# weaken($Lab::Bus::BusList{$self->type()}->{'default'});
+			 $Lab::Bus::BusList{$self->type()}->{'default'} = $self;
+			 weaken($Lab::Bus::BusList{$self->type()}->{'default'});
 		}
 	}
 
 	# set the connection $self->base_connection to the parameters required by IsoBus
+	$self->base_connection($self->config('base_connection')) if defined $self->config('base_connection');
+	$self->IsoEnableTermChar($self->config('IsoEnableTermChar')) if defined $self->config('IsoEnableTermChar');
+	$self->IsoTermChar($self->config('IsoTermChar')) if defined $self->config('IsoTermChar');
 	# clear the connection if possible
 
-#   19     # we need to set the following RS232 options: 9600baud, 8 data bits, 1 stop bit, no parity, no flow control
-#   20     # what is the read terminator? we assume CR=13 here, but this is not set in stone
-#   21     # write terminator should I think always be CR=13=0x0d
-#   22     
-#   23     my $status=Lab::VISA::viSetAttribute($self->{vi}->{instr}, $Lab::VISA::VI_ATTR_ASRL_BAUD, 9600);
-#   24     if ($status != $Lab::VISA::VI_SUCCESS) { die "Error while setting baud: $status";}
-#   25 
-#   26     $status=Lab::VISA::viSetAttribute($self->{vi}->{instr}, $Lab::VISA::VI_ATTR_TERMCHAR, 13);
-#   27     if ($status != $Lab::VISA::VI_SUCCESS) { die "Error while setting termchar: $status";}
-#   28 
-#   29     $status=Lab::VISA::viSetAttribute($self->{vi}->{instr}, $Lab::VISA::VI_ATTR_TERMCHAR_EN, 1);
-#   30     if ($status != $Lab::VISA::VI_SUCCESS) { die "Error while setting termchar enabled: $status";}
-#   31 
-#   32     $status=Lab::VISA::viSetAttribute($self->{vi}->{instr}, $Lab::VISA::VI_ATTR_ASRL_END_IN,   
-#              $Lab::VISA::VI_ASRL_END_TERMCHAR);
-#   33     if ($status != $Lab::VISA::VI_SUCCESS) { die "Error while setting end termchar: $status";}
-#   34 
-#   35     #  here we might still have to reinitialize the serial port to make the settings come into effect. how???
-
+# we need to set the following RS232 options: 9600baud, 8 data bits, 1 stop bit, no parity, no flow control
+# what is the read terminator? we assume CR=13 here, but this is not set in stone
+# write terminator should I think always be CR=13=0x0d
+	
 	return $self;
 }
 
@@ -81,6 +71,7 @@ sub connection_new { # @_ = ({ isobus_address => $isobus_address })
 	my $connection_handle=undef;
 	if (ref $_[0] eq 'HASH') { $args=shift } # try to be flexible about options as hash/hashref
 	else { $args={@_} }
+	
 
 	my $isobus_address = $args->{'isobus_address'};
 
@@ -104,7 +95,7 @@ sub connection_read { # @_ = ( $connection_handle, $args = { read_length, brutal
 	my $read_length = $args->{'read_length'} || $self->read_length();
 	my $result = undef;
 
-	$result=$self->base_connection->Read({
+	$result=$self->config('base_connection')->Read({
 		    brutal => $brutal, 
 		    read_length => $read_length,
 	});
@@ -120,10 +111,12 @@ sub connection_write { # @_ = ( $connection_handle, $args = { command, wait_stat
 	my $args = undef;
 	if (ref $_[0] eq 'HASH') { $args=shift } # try to be flexible about options as hash/hashref
 	else { $args={@_} }
-
+	
+	
 	my $command = $args->{'command'} || undef;
 
 	my $write_cnt = 0;
+	
 
 	if(!defined $command) {
 		Lab::Exception::CorruptParameter->throw(
@@ -131,11 +124,18 @@ sub connection_write { # @_ = ( $connection_handle, $args = { command, wait_stat
 		);
 	}
 	else {
-		$write_cnt=$self->base_connection->Write({
-			# build the format for an IsoBus command
-			command => sprintf("@%d%s\r",$connection_handle,$command),
-		});
-
+		if ($self->config("IsoEnableTermChar")){
+			$write_cnt=$self->base_connection->Write({
+				# build the format for an IsoBus command
+				command => sprintf("@%d%s%s",$connection_handle,$command,$self->IsoTermChar()),
+			});
+			
+		}
+		else
+		{$write_cnt=$self->base_connection->Write({
+				# build the format for an IsoBus command
+				command => sprintf("@%d%s",$connection_handle,$command)})
+		};
 	return $write_cnt;
 	}
 }
@@ -169,6 +169,13 @@ sub connection_query { # @_ = ( $connection_handle, $args = { command, read_leng
 	return $result;
 }
 
+sub connection_clear {
+	my $self = shift;
+	
+	$self->config('base_connection')->Clear();
+
+}
+
 
 
 
@@ -200,7 +207,61 @@ Lab::Bus::IsoBus - Oxford Instruments IsoBus bus
 
 =head1 SYNOPSIS
 
-soon
+This is the IsoBus bus class. Typically you create it implicit through instrument creation:
+
+  my $instrument = new Lab::Instrument::IPS({
+    BusType => 'IsoBus',
+	base_connection => new Lab::Bus::VISA_GPIB({ gpib_board => 0, gpib_address => 24}),
+	isobus_addres => 2,
+  }
+  
+=head1 METHODS
+
+=head2 connection_new
+
+  $isobus->connection_new({ resource_name => $isobus_address });
+
+Creates a new instrument handle for this bus.
+
+The handle is usually stored in an instrument object and given to connection_read, connection_write etc.
+to identify and handle the calling instrument:
+
+  $InstrumentHandle = $isobus->connection_new({ resource_name => $isobus_address });
+  $result = $isobus->connection_read($self->InstrumentHandle(), { options });
+
+See C<Lab::Instrument::Read()>.
+
+
+=head2 connection_write
+
+  $isobus->connection_write( $InstrumentHandle, { command => $command, wait_status => $wait_status } );
+
+Puts in front of the $command-string the isobus_adress, e.g. "@1$command". Passes the modified argument hash to the base_connection.
+For further information refer to the specific connection class of $base_connection.
+
+=head2 connection_read
+
+  $isobus->connection_read( $InstrumentHandle, { command => $command, read_length => $read_length, timeout => $seconds,  brutal => 0/1 } );
+
+Puts in front of the $command-string the isobus_adress, e.g. "@1$command". Passes the modified argument hash to the base_connection.
+For further information refer to the specific connection class of $base_connection.
+
+
+=head2 connection_clear
+
+  $isobus->connection_clear( $InstrumentHandle );
+
+Clears the specified connection $InstrumentHandle.
+
+
+
+=head2 connection_query
+
+  $isobus->connection_query( $InstrumentHandle, { command => $command, read_length => $read_length, wait_status => $wait_status, wait_query => $wait_query, brutal => 0/1 } );
+
+Puts in front of the $command-string the isobus_adress, e.g. "@1$command". Passes the modified argument hash to the base_connection.
+For further information refer to the specific connection class of $base_connection.
+
 
 =head1 SEE ALSO
 
@@ -227,3 +288,4 @@ terms as Perl itself.
 
 1;
 
+
