@@ -1,7 +1,7 @@
 #!/usr/bin/perl -w
 
 package Lab::Connection::IsoBus;
-our $VERSION = '3.11';
+our $VERSION = '3.20';
 
 use strict;
 use Lab::Bus::VISA;
@@ -30,6 +30,61 @@ sub new {
 	return $self;
 }
 
+sub _configurebus { # $self->setbus() create new or use existing bus
+	my $self=shift;
+	
+
+	my $base = $self->config('base_connection');
+	
+	# add predefined connection settings to connection config:
+	# no overwriting of user defined connection settings
+	
+	my $new_config = $base->config();
+	for my $key ( keys %{$self->config()} )
+		{
+		if ( not defined $base->config($key) )
+			{
+			$new_config->{$key} = $self->config($key);
+			}
+		}
+	$new_config->{'base_connection'} = undef; # aviod recursive definition of bas_connection
+	$base->config($new_config);
+	$self->config('base_connection')->_configurebus();
+	
+		
+}
+
+
+sub block_connection {
+	my $self = shift;
+	
+		
+	$self->{connection_blocked} = 1;
+	$self->{config}->{base_connection}->block_connection();
+	
+}
+
+sub unblock_connection {
+	my $self = shift;
+	
+	$self->{connection_blocked} = undef;
+	$self->{config}->{base_connection}->unblock_connection();
+	
+}
+
+sub is_blocked {
+	my $self = shift;
+	
+	if ( $self->{connection_blocked} == 1 or  $self->{config}->{base_connection}->is_blocked() )
+		{
+		return 1;
+		}
+	else
+		{
+		return 0;
+		}
+	
+}
 
 1;
 
