@@ -6,28 +6,29 @@
 # RS232 settings to VISA attributes, mostly).
 #
 
-# TODO: Access to GPIB VISA attributes, device clear, ...
 
-die "Work in progress\n";
+
 
 package Lab::Connection::VISA_RS232;
-our $VERSION = '3.11';
+our $VERSION = '3.20';
 
 use strict;
 use Lab::Bus::VISA;
-use Lab::Connection::RS232;
+use Lab::Connection;
 use Lab::Exception;
 
 
-our @ISA = ("Lab::Connection::RS232");
+our @ISA = ("Lab::Connection");
 
 our %fields = (
 	bus_class => 'Lab::Bus::VISA',
 	resource_name => undef,
+	brutal => 0,
+	timeout => 2,
 	wait_status=>0, # sec;
 	wait_query=>10e-6, # sec;
 	read_length=>1000, # bytes
-	port=> undef,
+	rs232_address => undef,
 );
 
 
@@ -35,9 +36,10 @@ sub new {
 	my $proto = shift;
 	my $class = ref($proto) || $proto;
 	my $twin = undef;
+		
 	my $self = $class->SUPER::new(@_); # getting fields and _permitted from parent class, parameter checks
 	$self->${\(__PACKAGE__.'::_construct')}(__PACKAGE__);
-
+	
 	return $self;
 }
 
@@ -56,20 +58,93 @@ sub _setbus {
 	#
 	# build VISA resource name
 	#
-	my $resource_name = $self->port();
+	if (not defined $self->rs232_address())
+		{
+		Lab::Exception::UndefinedField->throw(error => 'No rs232 address defined !');
+		}
+		
+	my $resource_name = "ASRL";
+	$resource_name .= $self->rs232_address();
 	$resource_name .= '::INSTR';
 	$self->resource_name($resource_name);
 	$self->config()->{'resource_name'} = $resource_name;
 	
 	# again, pass it all.
 	$self->connection_handle( $self->bus()->connection_new( $self->config() ));
-
-	# TODO: now we need to set all the RS232 communication parameters, since the VISA
-	# bus does not evaluate them by default. (compared to RS232 bus)
-
+	
+	
 	return $self->bus();
+	
 }
 
+sub _configurebus {
+	my $self = shift;
+	
+	
+	#
+	# set VISA Attributes:
+	#
+	
+	
+	# boudrate
+	$self->bus()->set_visa_attribute($self->connection_handle(), $Lab::VISA::VI_ATTR_ASRL_BAUD, $self->config()->{baudrate});
+	
+	# databits
+	$self->bus()->set_visa_attribute($self->connection_handle(), $Lab::VISA::VI_ATTR_ASRL_DATA_BITS, $self->config()->{databits});
+	
+	# parity
+	my $parity;
+	if ( $self->config()->{parity} eq 'none' ) { $parity = 0; }
+	elsif ( $self->config()->{parity} eq 'odd' ) { $parity = 1; }
+	elsif ( $self->config()->{parity} eq 'even' ) { $parity = 2; }
+	elsif ( $self->config()->{parity} eq 'mark' ) { $parity = 3; }
+	elsif ( $self->config()->{parity} eq 'space' ) { $parity = 4; }
+	else { $parity = $self->config()->{parity}; }	
+	$self->bus()->set_visa_attribute($self->connection_handle(), $Lab::VISA::VI_ATTR_ASRL_PARITY, $parity);
+	
+	# stop bits
+	my $stopbits;
+	if ( $self->config()->{stopbits} == 1 )
+		{
+		$stopbits =  $Lab::VISA::VI_ASRL_STOP_ONE;
+		}
+	elsif ( $self->config()->{stopbits} == 1.5 )
+		{
+		$stopbits =  $Lab::VISA::VI_ASRL_STOP_ONE5;
+		}
+	elsif ( $self->config()->{stopbits} == 2 )
+		{
+		$stopbits =  $Lab::VISA::VI_ASRL_STOP_TWO;
+		}
+	else
+		{
+		$stopbits = $self->config()->{stopbits};
+		}
+	$self->bus()->set_visa_attribute($self->connection_handle(), $Lab::VISA::VI_ATTR_ASRL_STOP_BITS, $stopbits);
+	
+	# termination character
+	if ( defined $self->config()->{termchar} )
+		{
+		if ($self->config()->{termchar} =~ m/^[0-9]+$/)
+		{
+			$self->config()->{termchar} = chr($self->config()->{termchar});
+		}
+		$self->bus()->set_visa_attribute($self->connection_handle(), $Lab::VISA::VI_ATTR_TERMCHAR_EN, $Lab::VISA::VI_TRUE);
+		$self->bus()->set_visa_attribute($self->connection_handle(), $Lab::VISA::VI_ATTR_TERMCHAR,  ord($self->config()->{termchar}));
+		}
+	else
+		{
+		$self->bus()->set_visa_attribute($self->connection_handle(), $Lab::VISA::VI_ATTR_TERMCHAR_EN, $Lab::VISA::VI_FALSE);
+		}
+	
+	# read timeout
+	$self->bus()->set_visa_attribute($self->connection_handle(), $Lab::VISA::VI_ATTR_TMO_VALUE, $self->config()->{timeout}*1e3);
+
+	
+}
+
+
+
 
 1;
 
