@@ -3,18 +3,21 @@ package Lab::Instrument;
 use strict;
 use warnings;
 
-our $VERSION = '3.11';
+our $VERSION = '3.20';
 
+use Lab::Generic;
 use Lab::Exception;
 use Lab::Connection;
 use Carp qw(cluck croak);
 use Data::Dumper;
 use Clone qw(clone);
+use Class::ISA qw(self_and_super_path);
+use Hook::LexWrap;
 
 use Time::HiRes qw (usleep sleep);
 use POSIX; # added for int() function
 
-our @ISA = ();
+our @ISA = ('Lab::Generic');
 
 our $AUTOLOAD;
 
@@ -30,7 +33,7 @@ our %fields = (
 	supported_connections => [ 'ALL' ],
 	# for connection default settings/user supplied settings. see accessor method.
 	connection_settings => {
-		connection_type => 'LinuxGPIB',	
+		timeout =>	1
 	},
 
 	# default device settings/user supplied settings. see accessor method.
@@ -41,7 +44,9 @@ our %fields = (
 		query_long_length => 10240, # bytes
 	},
 	
-	device_cache => {},
+	device_cache => {
+		'id' => undef
+	},
 
 	device_cache_order => [],
 
@@ -50,6 +55,9 @@ our %fields = (
 
 
 
+
+
+
 sub new { 
 	my $proto = shift;
 	my $class = ref($proto) || $proto;
@@ -57,10 +65,28 @@ sub new {
 	if (ref $_[0] eq 'HASH') { $config=shift }
 	else { $config={@_} }
 
-	my $self={};
-	bless ($self, $class);
+	
 
+	my $self = $class->SUPER::new(@_);
 	$self->${\(__PACKAGE__.'::_construct')}(__PACKAGE__);
+	
+	# wrap additional code for automatic cache-handling aroung all paramter set- and get-functions defined in %fields->{device_cache}
+	my @isa = Class::ISA::self_and_super_path($class);
+	my $flag = 0;
+	while (@isa)
+		{
+		my $isa = pop @isa;
+		if ( $flag == 1)
+			{
+			$self->_init_cache_handling($isa);
+			}
+		if ( $isa eq 'Lab::Instrument' )
+			{
+			$flag = 1;
+			}
+		
+		}
+	
 
 	$self->config($config);
 
@@ -68,6 +94,7 @@ sub new {
 	# In most inherited classes, configure() is run through _construct()
 	#
 	$self->${\(__PACKAGE__.'::configure')}($self->config()); # use local configure, not possibly overwritten one
+
 	if( $class eq __PACKAGE__ ) {
 		# _setconnection after providing $config - needed for direct instantiation of Lab::Instrument
 		$self->_setconnection();
@@ -76,12 +103,19 @@ sub new {
 	# digest parameters
 	$self->device_name($self->config('device_name')) if defined $self->config('device_name');
 	$self->device_comment($self->config('device_comment')) if defined $self->config('device_comment');
+	
+	
+	$self->register_instrument();
 
 	return $self;
 }
 
 
 
+
+
+
+
 #
 # Call this in inheriting class's constructors to conveniently initialize the %fields object data.
 #
@@ -137,6 +171,7 @@ sub _construct {	# _construct(__PACKAGE__);
 	#
 	# Also, other stuff that should only happen in the top level class instantiation can go here.
 	#
+	
 	if( $class eq $package && $class ne 'Lab::Instrument' ) {
 		$self->_setconnection();
 		
@@ -150,6 +185,211 @@ sub _construct {	# _construct(__PACKAGE__);
 }
 
 
+
+
+
+
+
+# this methode implements the cache-handling:
+# 
+# It will wrap all get- and set-functions for parameters initialized in $fields->{device_cache} with additional pre- and post-processing code.
+# If a get-function is called and read_mode == cache, the $self->{device_cache}->{parameter} will be returned immediately. The original get-function won't be executed in this case.
+# This behaviour can be disabled by setting the parmeter $self->{config}->{no_cache} = 1.
+# The return-value of the get-function will be cached in $self->{device_cache}in any case.
+#
+# Set-functions will automatically call the corresponding get-function in the post-processing section, in order to keep the cache up to date.
+# 
+# If a requestID has been set, only the get-function, which placed the request will be executed, while all others return the cache-value. Set-functions won't be executed at all.
+# 
+
+sub _init_cache_handling {
+	 my $self = shift;
+	 my $class = shift; 
+	 
+	 
+	 no strict 'refs';
+	 
+	 # avoid to redefine the subs twice
+	 if ( defined ${$class.'::MODIFIED'} )
+		{
+		return;
+		}
+		
+	my $fields = *${\($class.'::fields')}{HASH};
+	my @cache_params = keys %{$fields->{device_cache}};
+	
+	
+		
+	# wrap parameter function defined in %fields->{device_cache}:
+	foreach my $cache_param (@cache_params)
+		{
+		my $set_sub = "set_".$cache_param; 
+		my $get_sub = "get_".$cache_param; 
+		
+		my $get_methode = *{$class."::".$get_sub};
+		my $set_methode = *{$class."::".$set_sub};
+			
+		if (  $class->can("set_".$cache_param) and exists &$set_methode )
+			{
+			# wrap set-function: 
+			wrap ($class."::".$set_sub, 
+				# before set-functions is executed:
+				pre => sub {
+					my $self = shift;
+			
+					# read_mode handling: do not execute if request is set:
+					if (defined $self->{requestID} or $self->connection()->is_blocked() )
+						{
+						$_[-1] = 'connection blocked';
+						}
+					},
+				# after set-functions is executed:
+				post => sub {
+				
+					# call coresponding get-function in order to keep the cache up to date, if available
+					if ( $self->can($get_sub))
+						{
+						my $var = $self->$get_sub(); 
+						}
+					
+					});
+			}
+		
+		if (  $class->can("get_".$cache_param) and  exists &$get_methode  )
+			{	
+			my $parameter = $cache_param;
+			
+			
+			# wrap get-function: 
+			wrap ($class."::".$get_sub, 
+			
+				# before get-functions is executed:
+				pre => sub {
+					my $self = shift;									
+					
+					
+					# read_mode handling:
+					my @args = @_;
+					pop @args;
+					my ($read_mode, $tail) = $self->_check_args(\@args, ['read_mode']);
+					
+					
+					# do not read if request has been set. set read_mode to cache if cache is available	
+					if ( $self->connection()->is_blocked() == 1 )
+						{
+						if ( defined $self->{device_cache}->{$parameter} )
+								{								
+								$read_mode = 'cache';
+								}
+							else
+								{
+								$_[-1] = 'connection_blocked' ;
+								}
+						}
+
+					
+					if (defined $self->{requestID}) 
+						{
+						my ($package, $filename, $subroutine, $line) = split(/ /, $self->{requestID});
+						
+						if ( $subroutine ne $class."::".$get_sub )
+							{
+							if ( defined $self->{device_cache}->{$parameter} )
+								{
+								
+								$read_mode = 'cache';
+								}
+							else
+								{
+								$_[-1] = 'connection_blocked' ;
+								}
+							}
+						else
+							{
+							$read_mode = undef;
+							pop @_ ;
+							}
+							
+						}
+						
+					
+					
+					
+					# return cache value if read_mode is set to cache
+					if ( defined $read_mode and $read_mode eq 'cache' and defined $self->{device_cache}->{$parameter} and not $self->{config}->{no_cache})
+						{		
+						$_[-1] = $self->{device_cache}->{$parameter};
+						} 
+						
+					},
+				
+				# after get-functions is executed:
+				post => sub {	
+					
+					# refresh cache value
+					if ( not defined $_[-1] or ref ($_[-1]) eq 'Hook::LexWrap::Cleanup' )
+						{
+						return;
+						}			
+					else
+						{
+						$self->{device_cache}->{$parameter} = $_[-1];
+						}
+					});
+				
+			}
+		
+		}
+	 	
+		
+	# remeber that we have allready redefined the functions	
+	${$class.'::MODIFIED'} = 1;
+		
+	use strict 'refs';
+
+
+ 
+}
+
+
+
+sub register_instrument {
+	my $self = shift;
+	
+	push( @{Lab::Instrument::REGISTERED_INSTRUMENTS}, $self );
+
+
+}
+
+sub sprint_config {
+	my $self = shift;
+
+	
+	$Data::Dumper::Varname = "device_cache_";	
+	my $config = Dumper $self->device_cache();
+	
+	$config .= "\n";
+
+	
+	$Data::Dumper::Varname = "connection_settings_";
+	if (defined $self->connection()) {
+	$config .= Dumper $self->connection()->config();
+	}
+	return $config;
+	
+
+}
+
+
+
+
+
+
+
+
+
+
+
 sub _getset_key{
 	my $self = shift;
 	my $ckey = shift;
@@ -158,13 +398,13 @@ sub _getset_key{
 	if( !defined $self->device_cache()->{$ckey}  ) {
 		my $subname = 'get_' . $ckey;
 		Lab::Exception::CorruptParameter->throw("No get method defined for device_cache field $ckey! \n") if ! $self->can($subname);
-		$self->device_cache()->{$ckey} = $self->$subname( from_device => 1 );
+		my $result = $self->$subname( );
 		}
 	else {
 		my $subname = 'set_' . $ckey;
 		
 		Lab::Exception::CorruptParameter->throw("No set method defined for device_cache field $ckey!\n") if ! $self->can($subname);
-		$self->$subname($self->device_cache()->{$ckey});
+		my $result = $self->$subname($self->device_cache()->{$ckey});
 	}
 	
 }
@@ -195,11 +435,11 @@ sub _cache_init {
 				$self->_getset_key($ckey) if exists $ckeyhash{$ckey};			
 			}
 			# initialize all values not in device_cache_order
-			for my $ckey (@ckeys){
-				$self->_getset_key($ckey) if not exists $orderhash{$ckey};
-			}
+			#for my $ckey (@ckeys){
+			#	$self->_getset_key($ckey) if not exists $orderhash{$ckey};
+			#}
 		}
-		# no ordering required
+		# no ordering requestd
 		else{
 			for my $ckey ( @ckeys ) {
 				$self->_getset_key($ckey);	
@@ -224,7 +464,7 @@ sub device_sync{
 	
 			
 	if( $self->{'device_cache'}){
-		if($pref eq 'driver'){			
+		if($pref eq 'driver'){		
 			if($_[0]){
 				$self->${\('set_'.$_[0])}($self->{'device_cache'}->{$_[0]});
 				return 1;
@@ -300,14 +540,15 @@ sub _checkconnection { # Connection object or connection_type string (as in Lab:
 	my $found = 0;
 
 	$connection = ref($connection) || $connection;
-
+	
 	return 0 if ! defined $connection;
 
 	no strict 'refs';
 	if( grep(/^ALL$/, @{$self->supported_connections()}) == 1 ) {
 		return $connection;
-	}
-	else {
+	} elsif ( $connection->isa('Lab::Connection::DEBUG')) {
+		return $connection;
+	} else {
 		for my $conn_supp ( @{$self->supported_connections()} ) {
 			return $conn_supp if( $connection->isa('Lab::Connection::'.$conn_supp));
 		}
@@ -331,11 +572,13 @@ sub _setconnection { # $self->setconnection() create new or use existing connect
 	for my $setting_key ( keys %{$self->connection_settings()} ) {
 		$config->{$setting_key} = $self->connection_settings($setting_key) if ! defined $config->{$setting_key};
 	}
-
+	
 	# check the configuration hash for a valid connection object or connection type, and set the connection
 	if( defined($self->config('connection')) ) {
+	
 		if($self->_checkconnection($self->config('connection')) ) {
 			$self->connection($self->config('connection'));
+			
 		}
 		else { Lab::Exception::CorruptParameter->throw( error => "Received invalid connection object!\n" ); }
 	}
@@ -347,8 +590,15 @@ sub _setconnection { # $self->setconnection() create new or use existing connect
 
 		if( $connection_type !~ /^[A-Za-z0-9_\-\:]*$/ ) { Lab::Exception::CorruptParameter->throw( error => "Given connection type is does not look like a valid module name.\n"); };
 
-		if( $connection_type eq 'none' ) { return; };
-		# todo: allow this only if the device supports connection_type none
+		if( $connection_type eq 'none' ) {
+			if( grep(/^none$/, @{$self->supported_connections()}) == 1 ) {
+				return;
+			} else {
+				Lab::Exception::Error->throw(
+					error => 	"Sorry, this instrument cannot work without a connection.\n"
+				);
+			};
+		};
 
 		$full_connection = "Lab::Connection::" . $connection_type;
 		eval("require ${full_connection};");
@@ -374,6 +624,21 @@ sub _setconnection { # $self->setconnection() create new or use existing connect
 	}
 	else {
 		Lab::Exception::CorruptParameter->throw( error => "Neither a connection nor a connection type was supplied.\n");	}
+
+
+
+	# add predefined connection settings to connection config:
+	# no overwriting of user defined connection settings
+	my $new_config = $self->connection()->config();
+	for my $key ( keys %{$self->connection_settings()} )
+		{
+		if ( not defined $self->connection()->config($key) )
+			{
+			$new_config->{$key} = $self->connection_settings($key);
+			}
+		}
+	$self->connection()->config($new_config);
+	$self->connection()->_configurebus();
 }
 
 
@@ -412,9 +677,8 @@ sub check_errors {
 	my $self=shift;
 	my $command=shift;
 	my @errors=();
-	
-	if($self->get_status()->{'ERROR'}) {
-	
+		
+	if($self->get_status()->{'ERROR'}) {	
 		my ( $code, $message )  = $self->get_error();	
 		while( $code != 0 && $code != -1 ) {
 			push @errors, [$code, $message];
@@ -443,6 +707,18 @@ sub check_errors {
 # passing through generic write, read and query from the connection.
 #
 
+sub set_id {
+	my $self = shift;	
+	my ($id) = $self->_check_args( \@_, ['id'] );
+	$self->{'device_cache'}->{'id'} = $id;	
+	
+}
+
+sub get_id {
+	my $self = shift;
+	return $self->{'device_cache'}->{'id'};
+}
+
 sub write {
 	my $self=shift;
 	my $command= scalar(@_)%2 == 0 && ref $_[1] ne 'HASH' ? undef : shift;  # even sized parameter list and second parm no hashref? => Assume parameter hash
@@ -451,9 +727,11 @@ sub write {
 
 	$args->{'command'} = $command if defined $command;
 	
-	$self->connection()->Write($args);
+	my $result = $self->connection()->Write($args);
 	
 	$self->check_errors($args->{'command'}) if $args->{error_check};
+
+	return $result;
 }
 
 
@@ -461,29 +739,72 @@ sub read {
 	my $self=shift;
 	my $args = scalar(@_)%2==0 ? {@_} : ( ref($_[0]) eq 'HASH' ? $_[0] : undef );
 	Lab::Exception::CorruptParameter->throw( "Illegal parameter hash given!\n" ) if !defined($args);
-
+	
 	my $result = $self->connection()->Read($args);
 	$self->check_errors('Just a plain and simple read.') if $args->{error_check};
+	
+	$result =~ s/^[\r\t\n]+|[\r\t\n]+$//g;
 	return $result;
 }
 
 
-# query( $command, { channel => 1 })
-# query( $command, channel => 1 )
-# query({ command => $cmd, channel => 1 })
-# query( command => $cmd, channel => 1 )
+sub request {
+	my $self = shift;
+	my ($command, $args) = $self->parse_optional(@_);
+	my $read_mode = (defined $args->{'read_mode'}) ? $args->{'read_mode'} : 'device';
+	
+	
+	# generate requestID from caller:
+	my ($package, $filename, $line, $subroutine);
+	($package, $filename, $line, $subroutine) = caller(1);
+	($package, $filename, $line) = caller(0);
+	my $requestID = $package." ".$filename." ".$subroutine." ".$line;
+	
+	
+	# # avoid to return an undef value:
+	if ( $read_mode eq 'request' and not defined $self->{requestID} )
+		{
+		$self->write(@_);
+		$self->connection()->block_connection();
+		$self->{requestID} = $requestID;
+		return undef;
+		}
+	elsif ( defined $self->{requestID} and $self->{requestID} eq $requestID )
+		{
+		$self->connection()->unblock_connection();
+		$self->{requestID} = undef;
+		return $self->read(@_);
+		}
+	else
+		{
+		return $self->query(@_);
+		}
+		
+		
+	
+}
 
-# $time, $true_scalar
 
 sub query {
 	my $self=shift;
 	my ($command, $args) = $self->parse_optional(@_);
-
+	my $read_mode = (defined $args->{'read_mode'}) ? $args->{'read_mode'} : 'device';
 	$args->{'command'} = $command if defined $command;
-
+	
+	if ( not defined $args->{'command'} )
+		{
+		Lab::Exception::CorruptParameter->throw( "No 'command' given!\n" );
+		}
+	
+		
 	my $result = $self->connection()->Query($args);
 	$self->check_errors($args->{'command'}) if $args->{error_check};
-	return $result;
+	
+	$result =~ s/^[\r\t\n]+|[\r\t\n]+$//g;
+	return $result;	
+		
+			
+		
 }
 
 
@@ -617,6 +938,80 @@ sub connection_settings {
 	}
 }
 
+sub _check_args {
+	my $self = shift;
+	my $args = shift;
+	my $params = shift;
+	
+	my $arguments = {};
+
+	my $i = 0;
+	foreach my $arg (@{$args}) 
+	{
+		if ( ref($arg) ne "HASH" )
+			{
+			if ( defined @{$params}[$i] )
+				{
+				$arguments->{@{$params}[$i]} = $arg;				
+				}
+			$i++;
+			}
+		else
+			{
+			%{$arguments} = (%{$arguments}, %{$arg});
+			$i++;
+			}
+	}
+
+			
+	my @return_args = ();
+	
+	foreach my $param (@{$params}) 
+		{
+		if (exists $arguments->{$param}) 
+			{
+			push (@return_args, $arguments->{$param});
+			delete $arguments->{$param};
+			}
+		else
+			{
+			push (@return_args, undef);
+			}
+		}
+
+	foreach my $param ('from_device', 'from_cache') 	# Delete Standard option parameters from $arguments hash if not defined in device driver function
+		{
+		if (exists $arguments->{$param}) 
+			{
+			delete $arguments->{$param};
+			}
+		}
+		
+
+	push(@return_args, $arguments);
+	# if (scalar(keys %{$arguments}) > 0) 
+		# {
+		# my $errmess = "Unknown parameter given in $self :";
+		# while ( my ($k,$v) = each %{$arguments} ) 
+			# {
+			# $errmess .= $k." => ".$v."\t";
+			# }
+		# print Lab::Exception::Warning->new( error => $errmess);
+		# }
+		
+	if (wantarray)
+		{
+		return @return_args;
+		}
+	else
+		{
+		return $return_args[0];
+		}
+			
+	
+}
+	
+
 
 #
 # config gets it's own accessor - convenient access like $self->config('GPIB_Paddress') instead of $self->config()->{'GPIB_Paddress'}
@@ -947,6 +1342,17 @@ Uses get_error() to check the device for occured errors. Reads all present
 errors and throws a Lab::Exception::DeviceError. The list of errors, the device
 class and the last issued command(s) (if the script provided them) are enclosed.
 
+=head2 _check_args
+
+	my ($arg_1, $arg_2, ... ) = $instrument->_check_args(\@args, @arg_names );
+	
+	This methode is expected to be used to deal with the different possibilities a user can pass arguments when calling an instruments methode.
+	@args are the users arguments. The expected arguments of a specific methode may be $arg_1 and $arg_2.
+	Using the old style calling a methode, you have to pass the two arguments in the right order.
+	Using the new style, you can pass the arguments as a HASH using the arguments' names @arg_names. You don't have to take care of the right order in this case.
+	Furthermore the methode triggers a waring if it finds unknown parameter in the arguments hash ( this is to cathch typos in the script ).
+	
+
 =head1 CAVEATS/BUGS
 
 Probably many, with all the porting. This will get better.
@@ -978,6 +1384,7 @@ Probably many, with all the porting. This will get better.
            2010      Matthias Völker <mvoelker@cpan.org>
            2011      Florian Olbrich, Andreas K. Hüttel
 	   2012      Andreas K. Hüttel
+	   2013      Stefan Geissler, Christian Butschkow, Andreas K. Hüttel
 
 This library is free software; you can redistribute it and/or modify it under the same
 terms as Perl itself.
