@@ -3,7 +3,7 @@ package Lab::Instrument::YokogawaGS200;
 use strict;
 use warnings;
 
-our $VERSION = '3.11';
+our $VERSION = '3.20';
 
 use feature "switch";
 use Lab::Instrument;
@@ -13,7 +13,7 @@ use Lab::Instrument::Source;
 our @ISA=('Lab::Instrument::Source');
 
 our %fields = (
-	supported_connections => [ 'VISA_GPIB', 'GPIB', 'VISA', 'DEBUG' ],
+	supported_connections => [ 'VISA_GPIB', 'GPIB', 'VISA' ],
 
 	# default settings for the supported connections
 	connection_settings => {
@@ -194,7 +194,7 @@ sub program_halt{
 	$self->write("$cmd");
 }
 
-sub sweep {
+sub _sweep_to_level {
     my $self=shift;
     my $target=shift;
     my $time=shift;
@@ -631,7 +631,7 @@ The YokogawaGP200 class is a Source (L<Lab::Instrument::Source>)
  (c) 2004-2006 Daniel Schröer
  (c) 2007-2010 Daniel Schröer, Daniela Taubert, Andreas K. Hüttel, and others
  (c) 2011 Florian Olbrich, Andreas K. Hüttel
- (c) 2012 Alois Dirnaichner
+ (c) 2012-2013 Alois Dirnaichner, Andreas K. Hüttel
 
 This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
