@@ -1,6 +1,6 @@
 
 package Lab::Measurement;
-our $VERSION = '3.11';
+our $VERSION = '3.20';
 
 use strict;
 use warnings;
@@ -31,15 +31,35 @@ sub new {
         #axes           => [],
         #plots          => {},
         #constants      => [],
-        
+
         #live_plot      => '',
         #live_refresh   => '',
         #live_latest    => '',
 
 	#no_termcontrol =>0,
-        
+
         #writer_config  => {},
 
+    if (!defined($params{sample}))
+    {
+        Lab::Exception::CorruptParameter->throw( error => "Missing parameter in Lab::Mesaurement(): 'sample'\n" );
+    }
+    if (!defined($params{filename}) && !defined($params{filename_base}))
+    {
+        Lab::Exception::CorruptParameter->throw( error => "Missing parameter in Lab::Mesaurement(): 'filename' or 'filename_base'\n" );
+    }
+    if (!defined($params{columns}))
+    {
+        Lab::Exception::CorruptParameter->throw( error => "Missing parameter in Lab::Mesaurement(): 'columns'\n" );
+    }
+    if (!defined($params{axes}))
+    {
+        Lab::Exception::CorruptParameter->throw( error => "Missing parameter in Lab::Mesaurement(): 'axes'\n" );
+    }
+    if (!defined($params{plots}))
+    {
+        Lab::Exception::CorruptParameter->throw( error => "Missing parameter in Lab::Mesaurement(): 'plots'\n" );
+    }
     # initialize terminal if requested
     $self->{termctl}=0;
     if (! $params{no_termcontrol}) {
@@ -47,6 +67,12 @@ sub new {
 	$self->{termctl}=1;
     }
 
+#    $self->{itx_format}=0;
+#    if ($params{itx_format}) {
+#	$self->{itx_format}=1;
+#    }		
+
+
     # Filenamen finden
     if ($params{filename_base}) {
         my $fnb=$params{filename_base};
@@ -61,12 +87,13 @@ sub new {
 
     # Writer erzeugen, Log öffnen
     my $writer=new Lab::Data::Writer($params{filename},$params{writer_config});
+
     # header schreiben
     $writer->log_comment("Sample $params{sample}");
     $writer->log_comment($params{title});
     $writer->log_comment($params{description});
     $writer->log_comment("Recorded with Lab::Measurement $Lab::Measurement::VERSION");
-        
+
     # Meta erzeugen
     my $meta=new Lab::Data::Meta({
         data_complete           => 0,
@@ -77,8 +104,8 @@ sub new {
     });
     $meta->column($params{columns});
     $meta->axis($params{axes});
-    $meta->plot($params{plots});
-    $meta->constant($params{constants});
+    $meta->plot($params{plots}); 
+    $meta->constant($params{constants} || []); 
     my ($filename,$path,$suffix)=($writer->get_filename(),$writer->configure('output_meta_ext'));
     $meta->save("$path$filename.$suffix");
     
@@ -149,7 +176,11 @@ sub finish_measurement {
         $self->{live_plotter}->stop_live_plot();
         delete $self->{live_plotter};
     }
+
     delete $self->{writer};
+#    if($self->{itx_format}) {
+#    	$self->create_itx();
+#    }
     return delete $self->{meta};
 }
 
@@ -208,6 +239,37 @@ sub log {
     }
 }
 
+# sub create_itx{
+# 	my $self=shift;
+# 	    	#my $fh=$self->{writer};
+#     	#print $fh "END\r\n"
+# 		    	#itx header for IGOR
+# 	    my $itx_header= "IGOR\rWAVES/D";
+# 	    my $filestr=$self->{meta}->{data_file};
+# 	    my $path=$self->{meta}->{__abs_path};
+# 	    open my $fh, "$path$filestr";
+# 	    $filestr=~ s/\.\w*$//;
+# 	    open my $fh_itx, ">$path$filestr.itx";    
+# 	    #for my $fields_key ( keys %{$self->{_permitted}} )
+# 	    foreach my $value ( values($self->{meta}->{column}) ){	    
+# 	    	my $labelstr=$value->{label};
+# 	    	$itx_header.=sprintf("\t%s_%s",$filestr,$labelstr);
+# 	    }	
+# 	    $itx_header.="\nBEGIN\n";
+# 	    $fh_itx->write($itx_header);
+# 	    while  (defined(my $line=<$fh>)) {
+# 		    if ($line=~m/^[^\#]/){
+# 		    	$fh_itx->write($line)
+# 		    }else{
+# 		    	#print  "comment\n"
+# 		    }	
+# 	    }
+# 	    $fh_itx->write("\nEND\n");
+# 		#print $fh ($itx_header);	
+# }
+    
+	
+
 1;
 
 __END__
